@@ -70,6 +70,7 @@ class Session(dict):
         self.request = request
         self.data_dir = data_dir
         self.key = key
+        
         self.timeout = timeout
         self.use_cookies = use_cookies
         self.cookie_expires = cookie_expires
@@ -79,7 +80,7 @@ class Session(dict):
         self.secure = secure
         self.id = id
         self.accessed_dict = {}
-            
+        
         if self.use_cookies:
             cookieheader = request.get('cookie', '')
             if secret:
@@ -96,6 +97,7 @@ class Session(dict):
         self.is_new = self.id is None
         if self.is_new:
             self._create_id()
+            self['_accessed_time'] = self['_creation_time'] = time.time()
         else:
             try:
                 self.load()
@@ -111,6 +113,7 @@ class Session(dict):
                               getpid())).hexdigest(), 
         ).hexdigest()
         self.is_new = True
+        self.last_accessed = None
         if self.use_cookies:
             self.cookie[self.key] = self.id
             if self.cookie_domain:
@@ -175,6 +178,7 @@ class Session(dict):
         self.request['set_cookie'] = True
         
         self.namespace.acquire_read_lock()
+        timed_out = False
         try:
             self.clear()
             try:
@@ -195,14 +199,25 @@ class Session(dict):
                 }
                 self.is_new = True
             
-            if self.timeout is not None and now - session_data['_accessed_time'] > self.timeout:
-                self.invalidate()
+            if self.timeout is not None and \
+               now - session_data['_accessed_time'] > self.timeout:
+                timed_out= True
             else:
+                # Properly set the last_accessed time, which is different
+                # than the *currently* _accessed_time
+                if self.is_new or '_accessed_time' not in session_data:
+                    self.last_accessed = None
+                else:
+                    self.last_accessed = session_data['_accessed_time']
+                
+                # Update the current _accessed_time
                 session_data['_accessed_time'] = now
                 self.update(session_data)
                 self.accessed_dict = session_data.copy()
         finally:
             self.namespace.release_read_lock()
+        if timed_out:
+            self.invalidate()
     
     def save(self, accessed_only=False):
         """Saves the data for this session to persistent storage
@@ -381,6 +396,8 @@ class CookieSession(Session):
     
     def save(self, accessed_only=False):
         """Saves the data for this session to persistent storage"""
+        if accessed_only and self.is_new:
+            return
         if accessed_only:
             self.clear()
             self.update(self.accessed_dict)
@@ -524,7 +541,7 @@ class SessionObject(object):
         if self.__dict__['_params'].get('auto'):
             self._session().save()
         else:
-            if self.__dict__['_dirty']:
+            if self.__dict__.get('_dirty'):
                 self._session().save()
             else:
                 self._session().save(accessed_only=True)
